@@ -48,12 +48,7 @@ clients = [{
 
 backup = {
     'data': {
-        'directories': [
-            '/var/lib/containers/storage/volumes/nextcloud-volume-fbx/'
-        ],
-        'files': [
-            '/var/lib/plinth/backups-data/nextcloud-database.sql',
-            '/etc/redis/redis.conf'
-        ]
+        'directories': ['/var/lib/nextcloud/'],
+        'files': ['/var/lib/plinth/backups-data/nextcloud-database.sql']
     }
 }
