@@ -5,16 +5,6 @@ from django.utils.translation import gettext_lazy as _
 from plinth.clients import store_url
 
 clients = [
-    {
-        'name':
-            _('Tiny Tiny RSS (TTTRSS)'),
-        'platforms': [{
-            'type': 'store',
-            'os': 'android',
-            'store_name': 'f-droid',
-            'url': store_url('f-droid', 'org.fox.tttrss')
-        }]
-    },
     {
         'name':
             _('TTRSS-Reader'),
