@@ -1,7 +1,7 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
 """Package holding all the privileged actions outside of apps."""
 
-from .config import dropin_link, dropin_unlink
+from .config import dropin_is_valid, dropin_link, dropin_unlink
 from .packages import (filter_conffile_packages, install,
                        is_package_manager_busy, remove, update)
 from .service import (disable, enable, is_enabled, is_running, mask, reload,
@@ -11,5 +11,5 @@ __all__ = [
     'filter_conffile_packages', 'install', 'is_package_manager_busy', 'remove',
     'update', 'disable', 'enable', 'is_enabled', 'is_running', 'mask',
     'reload', 'restart', 'start', 'stop', 'try_restart', 'unmask',
-    'dropin_link', 'dropin_unlink'
+    'dropin_is_valid', 'dropin_link', 'dropin_unlink'
 ]
