@@ -15,7 +15,8 @@ from contextlib import contextmanager
 
 import pytest
 import requests
-from selenium.common.exceptions import (StaleElementReferenceException,
+from selenium.common.exceptions import (ElementClickInterceptedException,
+                                        StaleElementReferenceException,
                                         WebDriverException)
 from selenium.webdriver.support.ui import WebDriverWait
 
@@ -131,6 +132,10 @@ def wait_for_page_update(browser, timeout=300, expected_url=None):
     page_body = browser.find_by_tag('body').first
     try:
         yield
+    except ElementClickInterceptedException:
+        # When a element that is not visible is clicked, the click is ignored
+        # and we can't expect a page update.
+        raise
     except WebDriverException:
         # ignore a connection failure which may happen after web server restart
         pass
@@ -198,6 +203,17 @@ def download_file_outside_browser(url):
 ###########################
 # Form handling utilities #
 ###########################
+def click(browser, element):
+    """Click an element after scrolling it into view considering header."""
+    try:
+        element.click()
+    except ElementClickInterceptedException:
+        script = 'arguments[0].style.scrollMarginTop = "4.3125rem";' \
+            'arguments[0].scrollIntoView(true);'
+        browser.execute_script(script, element._element)
+        element.click()
+
+
 def submit(browser, element=None, form_class=None, expected_url=None):
     """Submit a specific form in the current page and wait for page change."""
     if not (element or form_class):
@@ -205,12 +221,13 @@ def submit(browser, element=None, form_class=None, expected_url=None):
 
     with wait_for_page_update(browser, expected_url=expected_url):
         if element:
-            element.click()
+            click(browser, element)
         elif form_class:
-            browser.find_by_css(
-                '.{} input[type=submit]'.format(form_class)).click()
+            browser.find_by_css(f'.{form_class} input[type=submit], '
+                                f'.{form_class} button[type=submit]').click()
         else:
-            browser.find_by_css('input[type=submit]').click()
+            browser.find_by_css(
+                'input[type=submit] button[type=submit]').click()
 
 
 def set_app_form_value(browser, app_id, element_id, value):
@@ -681,7 +698,7 @@ class BaseAppTests:
         """Install the app and set it up if needed."""
         install(session_browser, self.app_name)
 
-    @pytest.fixture(autouse=True)
+    @pytest.fixture(autouse=True, name='background')
     def fixture_background(self, session_browser):
         """Login, install, and enable the app."""
         login(session_browser)
